-- X-Perl UnitFrames
-- Author: Zek <Boodhoof-EU>
-- License: GNU GPL v3, 29 June 2007 (see LICENSE.txt)

local conf
XPerl_RequestConfig(function(new)
				conf = new
				if (XPerl_TargetTarget) then XPerl_TargetTarget.conf = conf.targettarget end
				if (XPerl_TargetTargetTarget) then XPerl_TargetTargetTarget.conf = conf.targettargettarget end
				if (XPerl_FocusTarget) then XPerl_FocusTarget.conf = conf.focustarget end
				if (XPerl_PetTarget) then XPerl_PetTarget.conf = conf.pettarget end
			end, "$Revision: 652 $")

local UnitName = UnitName
local UnitPower = UnitPower
local UnitHealth = UnitHealth
local UnitIsConnected = UnitIsConnected
local XPerl_Target_SetGray = XPerl_Target_SetGray
local XPerl_Target_UpdateLevel = XPerl_Target_UpdateLevel
local XPerl_Target_UpdatePVP = XPerl_Target_UpdatePVP
local XPerl_TargetTarget_Buff_UpdateAll = XPerl_TargetTarget_Buff_UpdateAll

local buffSetup

-- XPerl_TargetTarget_OnLoad
function XPerl_TargetTarget_OnLoad(self)
	self:RegisterForClicks("AnyUp")
	self:RegisterForDrag("LeftButton")
	XPerl_SetChildMembers(self)
	
	events = {"UNIT_AURA"};

	-- Events
	self:RegisterEvent("RAID_TARGET_UPDATE")
	if (self == XPerl_TargetTarget) then
		self.parentunit = "target"
		self.unit = "targettarget"
		self:RegisterEvent("PLAYER_TARGET_CHANGED")
		self:SetScript("OnUpdate", XPerl_TargetTarget_OnUpdate)
	elseif (self == XPerl_FocusTarget) then
		self.parentunit = "focus"
		self.unit = "focustarget"
		self:RegisterUnitEvent("UNIT_TARGET", "focus")
		self:SetScript("OnUpdate", XPerl_TargetTarget_OnUpdate)
	elseif (self == XPerl_PetTarget) then
		self.parentunit = "pet"
		self.unit = "pettarget"
		self:RegisterUnitEvent("UNIT_TARGET", "pet")
		self:SetScript("OnUpdate", XPerl_TargetTarget_OnUpdate)
	else
		self.parentunit = "targettarget"
		self.unit = "targettargettarget"
		self:SetScript("OnUpdate", XPerl_TargetTargetTarget_OnUpdate)
	end

	XPerl_SecureUnitButton_OnLoad(self, self.unit, XPerl_ShowGenericMenu)
	XPerl_SecureUnitButton_OnLoad(self.nameFrame, self.unit, XPerl_ShowGenericMenu)

	local BuffOnUpdate, DebuffOnUpdate, BuffUpdateTooltip, DebuffUpdateTooltip
	BuffUpdateTooltip = XPerl_Unit_SetBuffTooltip
	DebuffUpdateTooltip = XPerl_Unit_SetDeBuffTooltip

	if (buffSetup) then
		self.buffSetup = buffSetup
	else
		self.buffSetup = {
			buffScripts = {
				OnEnter = XPerl_Unit_SetBuffTooltip,
				OnUpdate = BuffOnUpdate,
				OnLeave = XPerl_PlayerTipHide,
			},
			debuffScripts = {
				OnEnter = XPerl_Unit_SetDeBuffTooltip,
				OnUpdate = DebuffOnUpdate,
				OnLeave = XPerl_PlayerTipHide,
			},
			updateTooltipBuff = BuffUpdateTooltip,
			updateTooltipDebuff = DebuffUpdateTooltip,
			debuffParent = true,
			debuffSizeMod = 0.2,
			debuffAnchor1 = function(self, b) b:SetPoint("TOPLEFT", 0, 0) end,
		}
		self.buffSetup.buffAnchor1 = self.buffSetup.debuffAnchor1
		buffSetup = self.buffSetup
	end

	self.targetname = ""
	self.time, self.targethp, self.targetpower, self.lastUpdate = 0, 0, 0, 0

	--XPerl_InitFadeFrame(self)
	XPerl_RegisterHighlight(self.highlight, 2)
	XPerl_RegisterPerlFrames(self, {self.nameFrame, self.statsFrame, self.levelFrame})

	if (XPerlDB) then
		self.conf = XPerlDB[self.unit]
	end
	if (self == XPerl_TargetTarget) then
		XPerl_RegisterOptionChanger(XPerl_TargetTarget_Set_Bits, "TargetTarget")
	end
	XPerl_Highlight:Register(XPerl_Target_HighlightCallback, self)

	if (XPerl_TargetTarget and XPerl_FocusTarget and XPerl_PetTarget and XPerl_TargetTargetTarget) then
		XPerl_TargetTarget_OnLoad = nil
	end
end

-------------------------
-- The Update Function --
-------------------------
-- XPerl_TargetTarget_RaidIconUpdate
local function XPerl_TargetTarget_RaidIconUpdate(self)
	local frameRaidIcon = self.nameFrame.raidIcon

	XPerl_Update_RaidIcon(frameRaidIcon, self.unit)

	frameRaidIcon:ClearAllPoints()
	if (conf.target.raidIconAlternate) then
		frameRaidIcon:SetHeight(16)
		frameRaidIcon:SetWidth(16)
		frameRaidIcon:SetPoint("CENTER", self.nameFrame, "TOPRIGHT", -5, -4)
	else
		frameRaidIcon:SetHeight(32)
		frameRaidIcon:SetWidth(32)
		frameRaidIcon:SetPoint("CENTER", self.nameFrame, "CENTER", 0, 0)
	end
end

-- XPerl_TargetTarget_Buff_UpdateAll
local function XPerl_TargetTarget_Buff_UpdateAll(self)
	self.conf.debuffs.enable = self.conf.buffs.enable
	XPerl_Targets_BuffUpdate(self)
end

-- XPerl_TargetTarget_UpdateDisplay
function XPerl_TargetTarget_UpdateDisplay(self,force)
	local unit = self.unit
	if (self.conf.enable and UnitExists(self.parentunit) and UnitIsConnected(unit)) then
		self.targetname = UnitName(unit)
		if (self.targetname ~= nil) then
			local t = GetTime()
			if (not force and t < self.lastUpdate + 0.3) then
				return
			end
			XPerl_Highlight:RemoveHighlight(self)
			self.lastUpdate = t

			-- Save these 3, so we know whether to update the frame later
			self.targethp = UnitHealth(unit)
			self.targetpower = UnitPower(unit)
			self.afk = UnitIsAFK(unit) and conf.showAFK

			XPerl_Unit_UpdateName(self)
			XPerl_Target_SetGray(self)
			XPerl_Target_UpdateLevel(self)
			XPerl_Target_UpdatePVP(self)
			XPerl_TargetTarget_Buff_UpdateAll(self)
			XPerl_TargetTarget_RaidIconUpdate(self)

			XPerl_UpdateSpellRange(self, unit)
			XPerl_Highlight:SetHighlight(self, UnitGUID(unit))
			return
		end
	end

	self.targetname = ""
	XPerl_Highlight:RemoveHighlight(self)
end

-- XPerl_TargetTarget_OnUpdate
function XPerl_TargetTarget_OnUpdate(self, elapsed)

	local unit = self.unit
	local newHP = UnitHealth(unit)
	local newPower = UnitPower(unit)
	local newAFK = UnitIsAFK(unit)

	if ((newHP ~= self.targethp) or (newPower ~= self.targetpower) or (newAFK ~= self.afk)) then
		XPerl_TargetTarget_UpdateDisplay(self)
	else
		self.time = elapsed + self.time
       	if (self.time >= 0.2) then
			XPerl_Target_Update_Combat(self)
			XPerl_Target_UpdateLevel(self)
			XPerl_Target_UpdatePVP(self)
			XPerl_TargetTarget_Buff_UpdateAll(self)
			XPerl_UpdateSpellRange(self, unit)
			XPerl_Highlight:SetHighlight(self, UnitGUID(unit))
			self.time = 0
		end
	end
end

-- XPerl_TargetTargetTarget_OnUpdate
function XPerl_TargetTargetTarget_OnUpdate(self, elapsed)

	local newName = UnitName(self.unit)

	if (not newName) then
		newName = ""
		newHP = 0
		newPower = 0
	end

	if (self == XPerl_TargetTargetTarget and newName ~= self.targetname) then
		XPerl_NoFadeBars(true)
		XPerl_TargetTarget_UpdateDisplay(self,true)
		XPerl_NoFadeBars()
		return
	end

	XPerl_TargetTarget_OnUpdate(self, elapsed)
end

-------------------
-- Event Handler --
-------------------
function XPerl_TargetTarget_OnEvent(self, event, unitID, ...)
	--print (self:GetName().."-"..event.."-"..(unitID or "nil"))
	if (event == "RAID_TARGET_UPDATE") then
		XPerl_TargetTarget_RaidIconUpdate(self)
	elseif ((event == "PLAYER_TARGET_CHANGED") and self == XPerl_TargetTarget) then
		XPerl_TargetTarget_UpdateDisplay(self, true)
	elseif (strfind(event, "^UNIT_")) then --(event == "UNIT_TARGET") then
		if (unitID == "target" and self == XPerl_TargetTarget) then
			XPerl_NoFadeBars(true)
			XPerl_TargetTarget_UpdateDisplay(self, true)
			XPerl_NoFadeBars()
		elseif (unitID == "focus" and self == XPerl_FocusTarget) then
			XPerl_NoFadeBars(true)
			XPerl_TargetTarget_UpdateDisplay(self, true)
			XPerl_NoFadeBars()
		elseif (unitID == "pet" and self == XPerl_PetTarget) then
			XPerl_NoFadeBars(true)
			XPerl_TargetTarget_UpdateDisplay(self, true)
			XPerl_NoFadeBars()
		end
	end
end

-- XPerl_TargetTarget_Update
function XPerl_TargetTarget_Update()
	local offset = -3
	if (self.conf.buffs.enable) then
		if (UnitExists("targettarget")) then
			if (XPerl_UnitBuff("targettarget", 1)) then
				if (offset == -3) then
					offset = 0
				end
				offset = offset + 20
				if (UnitBuff("targettarget", 9)) then
					offset = offset + 20
				end
			end
			if (XPerl_UnitDebuff("targettarget", 1)) then
				if (offset == -3) then
					offset = 0
				end
				offset = offset + 24
			end
		end
	end
end

-- EnableDisable
local function EnableDisable(self)
	if (self.conf.enable) then
		if (not self.virtual) then
			RegisterUnitWatch(self)
		end
	else
		UnregisterUnitWatch(self)
		self:Hide()
	end
end

-- XPerl_TargetTarget_SetWidth
function XPerl_TargetTarget_SetWidth(self)

	self.conf.size.width = max(0, self.conf.size.width or 0)
	local bonus = self.conf.size.width

	if (self.conf.percent) then
		self:SetWidth(160 + bonus)
		self.nameFrame:SetWidth(160 + bonus)
		self.statsFrame:SetWidth(160 + bonus)
		self.statsFrame.healthBar.percent:Show()
		self.statsFrame.manaBar.percent:Show()
	else
		self:SetWidth(128 + bonus)
		self.nameFrame:SetWidth(128 + bonus)
		self.statsFrame:SetWidth(128 + bonus)
		self.statsFrame.healthBar.percent:Hide()
		self.statsFrame.manaBar.percent:Hide()
	end

	self.conf.scale = self.conf.scale or 0.8
	self:SetScale(self.conf.scale)

	XPerl_SavePosition(self, true)

	XPerl_StatsFrameSetup(self)
end

-- Set
local function Set(self)
	self.nameText = self.nameFrame.text
	self.healthBar = self.statsFrame.healthBar
	self.healthBar.unit = self.unit
	self.manaBar = self.statsFrame.manaBar
	self.manaBar.unit = self.unit
	self.myHealPredictionBar = self.statsFrame.expectedHealth
	
	if (self.conf.level) then
		self.levelFrame:Show()
		self.levelFrame:SetWidth(27)
	else
		self.levelFrame:Hide()
	end

	if (self.conf.mana) then
		self.statsFrame.manaBar:Show()
		self.statsFrame:SetHeight(40)
	else
		self.statsFrame.manaBar:Hide()
		self.statsFrame:SetHeight(30)
	end

	if (self.conf.values) then
		self.statsFrame.healthBar.text:Show()
		self.statsFrame.manaBar.text:Show()
	else
		self.statsFrame.healthBar.text:Hide()
		self.statsFrame.manaBar.text:Hide()
	end

	self.buffFrame:ClearAllPoints()
	if (self.conf.buffs.above) then
		self.buffFrame:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 2, 0)
	else
		self.buffFrame:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 2, 0)
	end
	self.buffOptMix = nil
	self.conf.buffs.size = tonumber(self.conf.buffs.size) or 20

	XPerl_SetBuffSize(self)

	XPerl_TargetTarget_SetWidth(self)

	XPerl_ProtectedCall(EnableDisable, self)

	if (self:IsShown()) then
		XPerl_TargetTarget_UpdateDisplay(self, true)
	end
end

-- XPerl_TargetTarget_Set_Bits
function XPerl_TargetTarget_Set_Bits()
	if (not XPerl_TargetTarget) then
		return
	end

	if (conf.targettargettarget.enable) then
		if (not XPerl_TargetTargetTarget) then
			local ttt = CreateFrame("Button", "XPerl_TargetTargetTarget", UIParent, "XPerl_TargetTarget_Template")

			ttt:SetPoint("TOPLEFT", XPerl_TargetTarget.statsFrame, "TOPRIGHT", 5, 0)
		end
	end

	if (conf.focustarget.enable) then
		if (not XPerl_FocusTarget) then
			local ttt = CreateFrame("Button", "XPerl_FocusTarget", UIParent, "XPerl_TargetTarget_Template")

			ttt:SetPoint("TOPLEFT", XPerl_Focus.levelFrame, "TOPRIGHT", 5, 0)
		end
	end

	if (conf.pettarget.enable and XPerl_Player_Pet) then
		if (not XPerl_PetTarget) then
			local pt = CreateFrame("Button", "XPerl_PetTarget", XPerl_Player_Pet, "XPerl_TargetTarget_Template")

			pt:SetPoint("BOTTOMLEFT", XPerl_Player_Pet.statsFrame, "BOTTOMRIGHT", 5, 0)
		end
	end

	Set(XPerl_TargetTarget)
	if (XPerl_TargetTargetTarget) then
		Set(XPerl_TargetTargetTarget)
	end
	if (XPerl_FocusTarget) then
		Set(XPerl_FocusTarget)
	end
	if (XPerl_PetTarget) then
		Set(XPerl_PetTarget)
	end
end
